@@ -183,7 +183,7 @@ static dvd_input_t file_open(const char *target)
 /**
  * return the last error message
  */
-static char *file_error(dvd_input_t dev)
+static char *file_error(dvd_input_t dev UNUSED)
 {
   /* use strerror(errno)? */
   return (char *)"unknown error";
@@ -207,7 +207,7 @@ static int file_seek(dvd_input_t dev, int blocks)
 /**
  * set the block for the beginning of a new title (key).
  */
-static int file_title(dvd_input_t dev, int block)
+static int file_title(dvd_input_t dev UNUSED, int block UNUSED)
 {
   return -1;
 }
@@ -215,16 +215,16 @@ static int file_title(dvd_input_t dev, int block)
 /**
  * read data from the device.
  */
-static int file_read(dvd_input_t dev, void *buffer, int blocks, int flags)
+static int file_read(dvd_input_t dev, void *buffer, int blocks,
+		     int flags UNUSED)
 {
-  size_t len;
-  ssize_t ret;
+  size_t len, bytes;
 
   len = (size_t)blocks * DVD_VIDEO_LB_LEN;
+  bytes = 0;
 
   while(len > 0) {
-
-    ret = read(dev->fd, buffer, len);
+    ssize_t ret = read(dev->fd, ((char*)buffer) + bytes, len);
 
     if(ret < 0) {
       /* One of the reads failed, too bad.  We won't even bother
@@ -236,14 +236,15 @@ static int file_read(dvd_input_t dev, void *buffer, int blocks, int flags)
     if(ret == 0) {
       /* Nothing more to read.  Return all of the whole blocks, if any.
        * Adjust the file position back to the previous block boundary. */
-      size_t bytes = (size_t)blocks * DVD_VIDEO_LB_LEN - len;
       off_t over_read = -(bytes % DVD_VIDEO_LB_LEN);
-      /*off_t pos =*/ lseek(dev->fd, over_read, SEEK_CUR);
-      /* should have pos % 2048 == 0 */
+      off_t pos = lseek(dev->fd, over_read, SEEK_CUR);
+      if(pos % 2048 != 0)
+        fprintf( stderr, "libdvdread: lseek not multiple of 2048! Something is wrong!\n" );
       return (int) (bytes / DVD_VIDEO_LB_LEN);
     }
 
     len -= ret;
+    bytes += ret;
   }
 
   return blocks;
