@@ -450,6 +450,7 @@ dvd_reader_t *DVDOpen( const char *ppath )
         if( retval == -1 ) {
           goto DVDOpen_error;
         }
+        free(path_copy);
         path_copy = new_path;
         new_path = NULL;
       }
@@ -603,14 +604,11 @@ dvd_reader_t *DVDOpen( const char *ppath )
 DVDOpen_error:
   /* If it's none of the above, screw it. */
   fprintf( stderr, "libdvdread: Could not open %s\n", path );
-  if( path != NULL )
-    free( path );
-  if ( path_copy != NULL )
-    free( path_copy );
+  free( path );
+  free( path_copy );
   if ( cdir >= 0 )
     close( cdir );
-  if ( new_path != NULL )
-    free( new_path );
+  free( new_path );
   return NULL;
 }
 
@@ -682,7 +680,6 @@ static int findDirFile( const char *path, const char *file, char *filename )
 
 static int findDVDFile( dvd_reader_t *dvd, const char *file, char *filename )
 {
-  char video_path[ PATH_MAX + 1 ];
   const char *nodirfile;
   int ret;
 
@@ -695,6 +692,8 @@ static int findDVDFile( dvd_reader_t *dvd, const char *file, char *filename )
 
   ret = findDirFile( dvd->path_root, nodirfile, filename );
   if( ret < 0 ) {
+    char video_path[ PATH_MAX + 1 ];
+
     /* Try also with adding the path, just in case. */
     sprintf( video_path, "%s/VIDEO_TS/", dvd->path_root );
     ret = findDirFile( video_path, nodirfile, filename );
@@ -814,7 +813,6 @@ static dvd_file_t *DVDOpenVOBPath( dvd_reader_t *dvd, int title, int menu )
   char full_path[ PATH_MAX + 1 ];
   struct stat fileinfo;
   dvd_file_t *dvd_file;
-  int i;
 
   dvd_file = malloc( sizeof( dvd_file_t ) );
   if( !dvd_file ) return NULL;
@@ -857,6 +855,8 @@ static dvd_file_t *DVDOpenVOBPath( dvd_reader_t *dvd, int title, int menu )
     dvd_file->filesize = dvd_file->title_sizes[ 0 ];
 
   } else {
+    int i;
+
     for( i = 0; i < TITLES_MAX; ++i ) {
 
       sprintf( filename, "VTS_%02i_%i.VOB", title, i + 1 );
@@ -936,10 +936,10 @@ dvd_file_t *DVDOpenFile( dvd_reader_t *dvd, int titlenum,
 
 void DVDCloseFile( dvd_file_t *dvd_file )
 {
-  int i;
-
   if( dvd_file && dvd_file->dvd ) {
     if( !dvd_file->dvd->isImageFile ) {
+      int i;
+
       for( i = 0; i < TITLES_MAX; ++i ) {
         if( dvd_file->title_devs[ i ] ) {
           dvdinput_close( dvd_file->title_devs[i] );
@@ -1056,7 +1056,6 @@ int DVDFileStat( dvd_reader_t *dvd, int titlenum,
                  dvd_read_domain_t domain, dvd_stat_t *statbuf )
 {
   char filename[ MAX_UDF_FILE_NAME_LEN ];
-  char full_path[ PATH_MAX + 1 ];
   struct stat fileinfo;
   uint32_t size;
 
@@ -1112,6 +1111,8 @@ int DVDFileStat( dvd_reader_t *dvd, int titlenum,
       return 0;
     }
   } else {
+    char full_path[ PATH_MAX + 1 ];
+
     if( findDVDFile( dvd, filename, full_path ) ) {
       if( stat( full_path, &fileinfo ) < 0 )
         fprintf( stderr, "libdvdread: Can't stat() %s.\n", filename );
@@ -1185,7 +1186,7 @@ static int DVDReadBlocksPath( const dvd_file_t *dvd_file, unsigned int offset,
       if( ( offset + block_count ) <= dvd_file->title_sizes[ i ] ) {
         off = dvdinput_seek( dvd_file->title_devs[ i ], (int)offset );
         if( off < 0 || off != (int)offset ) {
-          fprintf( stderr, "libdvdread: Can't seek to block %d\n",
+          fprintf( stderr, "libdvdread: Can't seek to block %u\n",
                    offset );
           return off < 0 ? off : 0;
         }
@@ -1200,7 +1201,7 @@ static int DVDReadBlocksPath( const dvd_file_t *dvd_file, unsigned int offset,
         /* Read part 1 */
         off = dvdinput_seek( dvd_file->title_devs[ i ], (int)offset );
         if( off < 0 || off != (int)offset ) {
-          fprintf( stderr, "libdvdread: Can't seek to block %d\n",
+          fprintf( stderr, "libdvdread: Can't seek to block %u\n",
                    offset );
           return off < 0 ? off : 0;
         }
@@ -1360,7 +1361,7 @@ ssize_t DVDFileSize( dvd_file_t *dvd_file )
 
 int DVDDiscID( dvd_reader_t *dvd, unsigned char *discid )
 {
-  struct md5_ctx ctx;
+  struct md5_s ctx;
   int title;
   int title_sets;
   int nr_of_files = 0;
@@ -1385,7 +1386,7 @@ int DVDDiscID( dvd_reader_t *dvd, unsigned char *discid )
 
   /* Go through the first IFO:s, in order, up until the tenth,
    * and md5sum them, i.e  VIDEO_TS.IFO and VTS_0?_0.IFO */
-  md5_init_ctx( &ctx );
+  InitMD5( &ctx );
   for( title = 0; title < title_sets; title++ ) {
     dvd_file_t *dvd_file = DVDOpenFile( dvd, title, DVD_READ_INFO_FILE );
     if( dvd_file != NULL ) {
@@ -1410,14 +1411,15 @@ int DVDDiscID( dvd_reader_t *dvd, unsigned char *discid )
           return -1;
       }
 
-      md5_process_bytes( buffer, file_size,  &ctx );
+      AddMD5( &ctx, buffer, file_size );
 
       DVDCloseFile( dvd_file );
       free( buffer_base );
       nr_of_files++;
     }
   }
-  md5_finish_ctx( &ctx, discid );
+  EndMD5( &ctx );
+  memcpy( discid, ctx.buf, 16 );
   if(!nr_of_files)
     return -1;
 
